;; Generated package description from captain.el  -*- no-byte-compile: t -*-
(define-package "captain" "1.0.3.0.20221212.222029" "CAPiTalization is Automatic IN emacs" 'nil :commit "6481b7eea824a42234d8070d8e26fd04c1e04878" :url "https://elpa.gnu.org/packages/captain.html" :authors '(("Ian Dunn" . "dunni@gnu.org")) :maintainer '("Ian Dunn" . "dunni@gnu.org") :keywords '("editing"))
